pub type ThresholdConditionCache = HashMap<*const BlockIndex, ThresholdState>;
| A map that gives the state for blocks whose | height is a multiple of Period(). | | The map is indexed by the block’s parent, | however, so all keys in the map will either be | nullptr or a block with (height + 1) % Period() | == 0.