Struct bitcoin_txmempool::TxMemPoolInner
source · pub struct TxMemPoolInner {Show 14 fields
pub total_tx_size: u64,
pub total_fee: Amount,
pub cached_inner_usage: u64,
pub last_rolling_fee_update: Arc<Mutex<i64>>,
pub block_since_last_rolling_fee_bump: AtomicBool,
pub rolling_minimum_fee_rate: AtomicF64,
pub epoch: Arc<Mutex<Epoch>>,
pub sequence_number: RefCell<u64>,
pub is_loaded: bool,
pub map_tx: TxMemPoolIndexedTransactionSet,
pub tx_hashes: Vec<(u256, TxMemPoolTxIter)>,
pub unbroadcast_txids: HashSet<u256>,
pub map_next_tx: IndirectMap<OutPoint, Arc<Transaction>>,
pub map_deltas: HashMap<u256, Amount>,
}Fields
total_tx_size: u64| sum of all mempool tx’s virtual sizes. | Differs from serialized tx size since | witness data is discounted. Defined | in BIP 141. |
total_fee: Amount| sum of all mempool tx’s fees (NOT modified | fee) |
cached_inner_usage: u64| sum of dynamic memory usage of all the | map elements (NOT the maps themselves) |
last_rolling_fee_update: Arc<Mutex<i64>>block_since_last_rolling_fee_bump: AtomicBoolrolling_minimum_fee_rate: AtomicF64| minimum fee to get into the pool, decreases | exponentially |
epoch: Arc<Mutex<Epoch>>sequence_number: RefCell<u64>| In-memory counter for external mempool | tracking purposes. | | This number is incremented once every time | a transaction is added or removed from the | mempool for any reason.
is_loaded: boolmap_tx: TxMemPoolIndexedTransactionSettx_hashes: Vec<(u256, TxMemPoolTxIter)>| All tx witness hashes/entries in mapTx, | in random order |
unbroadcast_txids: HashSet<u256>| Track locally submitted transactions | to periodically retry initial broadcast. |
map_next_tx: IndirectMap<OutPoint, Arc<Transaction>>map_deltas: HashMap<u256, Amount>Auto Trait Implementations
impl !RefUnwindSafe for TxMemPoolInner
impl Send for TxMemPoolInner
impl !Sync for TxMemPoolInner
impl Unpin for TxMemPoolInner
impl !UnwindSafe for TxMemPoolInner